==> scalus_3-0.7.0-sources.jar: scalus/examples/PubKeyValidator.scala <==
package scalus.examples

import scalus.Compile
import scalus.Compiler.fieldAsData
import scalus.builtin
import scalus.builtin.Builtins.*
import scalus.builtin.ByteString
import scalus.builtin.ByteString.given
import scalus.builtin.Data

@Compile
object PubKeyValidator {
    def validator(datum: Unit, redeemer: Unit, ctx: Data) = {
        val txinfo = unConstrData(unConstrData(ctx).snd.head).snd
        val signatories = unListData(txinfo.tail.tail.tail.tail.tail.tail.tail.head)

        def findSignatureOrFail(sigs: builtin.List[Data]): Unit =
            if signatories.isEmpty then throw new RuntimeException("Signature not found")
            else if unBData(signatories.head) == hex"deadbeef"
            then ()
            else findSignatureOrFail(signatories.tail)

        findSignatureOrFail(signatories)
    }

    inline def validatorV2(inline pubKey: ByteString)(datum: Unit, redeemer: Unit, ctx: Data) = {
        import scalus.ledger.api.v2.ScriptContext
        val signatories = unListData(fieldAsData[ScriptContext](_.txInfo.signatories)(ctx))
        trace("got signatories")(())

        def findSignatureOrFail(sigs: builtin.List[Data]): Unit =
            trace("check sigs")(())
            if signatories.isEmpty then throw new RuntimeException("Signature not found")
            else if unBData(signatories.head) == pubKey
            then
                trace("found!")(())
                ()
            else
                trace("nope")(())
                findSignatureOrFail(signatories.tail)

        findSignatureOrFail(signatories)
    }
}

==> scalus_3-0.8.0-sources.jar: scalus/examples/PubKeyValidator.scala <==
package scalus.examples

import scalus.Compile
import scalus.builtin
import scalus.builtin.Builtins.*
import scalus.builtin.ByteString
import scalus.builtin.ByteString.given
import scalus.builtin.Data
import scalus.prelude.Prelude.log

@Compile
object PubKeyValidator {
    def validator(datum: Unit, redeemer: Unit, ctx: Data) = {
        val txinfo = unConstrData(unConstrData(ctx).snd.head).snd
        val signatories = unListData(txinfo.tail.tail.tail.tail.tail.tail.tail.head)

        def findSignatureOrFail(sigs: builtin.List[Data]): Unit =
            if signatories.isEmpty then throw new RuntimeException("Signature not found")
            else if unBData(signatories.head) == hex"deadbeef"
            then ()
            else findSignatureOrFail(signatories.tail)

        findSignatureOrFail(signatories)
    }

    inline def validatorV2(inline pubKey: ByteString)(datum: Unit, redeemer: Unit, ctx: Data) = {
        import scalus.ledger.api.v2.ScriptContext
        val signatories = ctx.field[ScriptContext](_.txInfo.signatories).toList
        log("got signatories")

        def findSignatureOrFail(sigs: builtin.List[Data]): Unit =
            log("check sigs")
            if signatories.isEmpty then throw new RuntimeException("Signature not found")
            else if signatories.head.toByteString == pubKey
            then log("found!")
            else trace("nope")(findSignatureOrFail(signatories.tail))

        findSignatureOrFail(signatories)
    }

    inline def validatorV3(inline pubKey: ByteString)(ctx: Data) = {
        import scalus.ledger.api.v3.ScriptContext
        val signatories = ctx.field[ScriptContext](_.txInfo.signatories).toList
        log("got signatories")

        def findSignatureOrFail(sigs: builtin.List[Data]): Unit =
            log("check sigs")
            if signatories.isEmpty then throw new RuntimeException("Signature not found")
            else if signatories.head.toByteString == pubKey
            then log("found!")
            else trace("nope")(findSignatureOrFail(signatories.tail))

        findSignatureOrFail(signatories)
    }
}
